/*!
 * @file    	main.c
 * @brief	 	Main program body
 *
 * @details		
 *
 * @version		V1.0.0
 * @author  	
 * @date    	2019-9-3
 *
 */ 

#include "apm32f10x.h"
#include "apm32_mini.h"
#include "main.h"

/* Delay timing */ 
static __IO uint32_t TimingDelay;
/* Inserts a delay time */ 
void Delay(__IO uint32_t nTime);

/*!
 * @brief		Main program
 *
 * @param		None
 *
 * @retval		None
 *
 * @see 
 *
 * @note 
 */	  
int main(void)
{
	/* LED Init */
	APM_MINI_LedInit(LED2);
	APM_MINI_LedInit(LED3);
	
	/* Turn on LED2 */
	APM_MINI_LedOn(LED2);	
	/* Turn off LED3 */
	APM_MINI_LedOff(LED3);
	
	if(SysTick_Config(SystemCoreClock / 1000))
	{ 
		/* Capture error */ 
		while (1);
	}
	
	while (1)
	{
		Delay(500);
		
		APM_MINI_LedToggle(LED2);	
		APM_MINI_LedToggle(LED3);
	}
}

/*!
 * @brief		Inserts a delay time
 *
 * @param		nTime	specifies the delay time length, in milliseconds
 *
 * @retval		None
 *
 * @see 
 *
 * @note 
 */
void Delay(__IO uint32_t nTime)
{ 
	TimingDelay = nTime;

	while(TimingDelay != 0);
}

/*!
 * @brief		Decrements the TimingDelay variable
 *
 * @param		None
 *
 * @retval		None
 *
 * @see 
 *
 * @note 
 */
void TimingDelay_Decrement(void)
{
	if (TimingDelay != 0x00)
	{ 
		TimingDelay--;
	}
}


