/*!
 * @file    	apm32f10x.h
 * @brief	 	CMSIS Cortex-M3 Device Peripheral Access Layer Header File.
 *
 * @details		This file contains all the peripheral register's definitions, bits definitions and memory mapping
 *
 * @version		V1.0.0
 * @author  	
 * @date    	2019-8-5
 *
 */

#ifndef __APM32F10x_H
#define __APM32F10x_H

#ifdef __cplusplus
 extern "C" {
#endif 
  
/*!
 *	APM32F10X_LD: APM32 Low density devices
 *	APM32F10X_MD: APM32 Medium density devices
 *  APM32F10X_HD: APM32 High density devices
 */
#if !defined (APM32F10X_LD)  && !defined (APM32F10X_MD) && !defined (APM32F10X_HD)
#error "Please select a the target APM32F10x device used in your application (in apm32f10x.h file)"
#endif

/**
 * @brief Define Value of the External oscillator in Hz
  */           
#ifndef  HXT_VALUE 
 #define HXT_VALUE				((uint32_t)8000000)
#endif

/** Time out for HXT start up */
#define HXT_STARTUP_TIMEOUT		((uint16_t)0x0500)

/** Value of the Internal oscillator in Hz */
#define HIRC_VALUE    			((uint32_t)8000000)

/**
 * @brief APM32F10x Standard Peripheral Library version number
   */
#define __APM32F10X_STDPERIPH_VERSION_MAIN   (0x01) /*!< [31:24] main version */                                  
#define __APM32F10X_STDPERIPH_VERSION_SUB1   (0x00) /*!< [23:16] sub1 version */
#define __APM32F10X_STDPERIPH_VERSION_SUB2   (0x00) /*!< [15:8]  sub2 version */
#define __APM32F10X_STDPERIPH_VERSION_RC     (0x00) /*!< [7:0]  release candidate */ 
#define __APM32F10X_STDPERIPH_VERSION       ( (__APM32F10X_STDPERIPH_VERSION_MAIN << 24)\
                                             |(__APM32F10X_STDPERIPH_VERSION_SUB1 << 16)\
                                             |(__APM32F10X_STDPERIPH_VERSION_SUB2 << 8)\
                                             |(__APM32F10X_STDPERIPH_VERSION_RC))


/** APM32 devices does not provide an MPU */
 #define __MPU_PRESENT				0
/** APM32 uses 4 Bits for the Priority Levels  */
#define __NVIC_PRIO_BITS          	4
/** Set to 1 if different SysTick Config is used */
#define __Vendor_SysTickConfig    	0

/**
 * @brief APM32F10x Interrupt Number Definition, according to the selected device 
 *        in @ref Library_configuration_section 
 */
typedef enum IRQn
{
/******  Cortex-M3 Processor Exceptions Numbers ***************************************************/
  NonMaskableInt_IRQn         = -14,    /*!< 2 Non Maskable Interrupt                             */
  MemoryManagement_IRQn       = -12,    /*!< 4 Cortex-M3 Memory Management Interrupt              */
  BusFault_IRQn               = -11,    /*!< 5 Cortex-M3 Bus Fault Interrupt                      */
  UsageFault_IRQn             = -10,    /*!< 6 Cortex-M3 Usage Fault Interrupt                    */
  SVCall_IRQn                 = -5,     /*!< 11 Cortex-M3 SV Call Interrupt                       */
  DebugMonitor_IRQn           = -4,     /*!< 12 Cortex-M3 Debug Monitor Interrupt                 */
  PendSV_IRQn                 = -2,     /*!< 14 Cortex-M3 Pend SV Interrupt                       */
  SysTick_IRQn                = -1,     /*!< 15 Cortex-M3 System Tick Interrupt                   */

/******  APM32 specific Interrupt Numbers *********************************************************/
  WWDG_IRQn                   = 0,      /*!< Window WatchDog Interrupt                            */
  PVD_IRQn                    = 1,      /*!< PVD through EXTI Line detection Interrupt            */
  TAMPER_IRQn                 = 2,      /*!< Tamper Interrupt                                     */
  RTC_IRQn                    = 3,      /*!< RTC global Interrupt                                 */
  FLASH_IRQn                  = 4,      /*!< FLASH global Interrupt                               */
  RCC_IRQn                    = 5,      /*!< RCC global Interrupt                                 */
  EXTI0_IRQn                  = 6,      /*!< EXTI Line0 Interrupt                                 */
  EXTI1_IRQn                  = 7,      /*!< EXTI Line1 Interrupt                                 */
  EXTI2_IRQn                  = 8,      /*!< EXTI Line2 Interrupt                                 */
  EXTI3_IRQn                  = 9,      /*!< EXTI Line3 Interrupt                                 */
  EXTI4_IRQn                  = 10,     /*!< EXTI Line4 Interrupt                                 */
  DMA1_Channel1_IRQn          = 11,     /*!< DMA1 Channel 1 global Interrupt                      */
  DMA1_Channel2_IRQn          = 12,     /*!< DMA1 Channel 2 global Interrupt                      */
  DMA1_Channel3_IRQn          = 13,     /*!< DMA1 Channel 3 global Interrupt                      */
  DMA1_Channel4_IRQn          = 14,     /*!< DMA1 Channel 4 global Interrupt                      */
  DMA1_Channel5_IRQn          = 15,     /*!< DMA1 Channel 5 global Interrupt                      */
  DMA1_Channel6_IRQn          = 16,     /*!< DMA1 Channel 6 global Interrupt                      */
  DMA1_Channel7_IRQn          = 17,     /*!< DMA1 Channel 7 global Interrupt                      */

#ifdef APM32F10X_LD
  ADC1_2_IRQn                 = 18,     /*!< ADC1 and ADC2 global Interrupt                       */
  USB_HP_CAN1_TX_IRQn         = 19,     /*!< USB Device High Priority or CAN1 TX Interrupts       */
  USB_LP_CAN1_RX0_IRQn        = 20,     /*!< USB Device Low Priority or CAN1 RX0 Interrupts       */
  CAN1_RX1_IRQn               = 21,     /*!< CAN1 RX1 Interrupt                                   */
  CAN1_SCE_IRQn               = 22,     /*!< CAN1 SCE Interrupt                                   */
  EXTI9_5_IRQn                = 23,     /*!< External Line[9:5] Interrupts                        */
  TIM1_BRK_IRQn               = 24,     /*!< TIM1 Break Interrupt                                 */
  TIM1_UP_IRQn                = 25,     /*!< TIM1 Update Interrupt                                */
  TIM1_TRG_COM_IRQn           = 26,     /*!< TIM1 Trigger and Commutation Interrupt               */
  TIM1_CC_IRQn                = 27,     /*!< TIM1 Capture Compare Interrupt                       */
  TIM2_IRQn                   = 28,     /*!< TIM2 global Interrupt                                */
  TIM3_IRQn                   = 29,     /*!< TIM3 global Interrupt                                */
  I2C1_EV_IRQn                = 31,     /*!< I2C1 Event Interrupt                                 */
  I2C1_ER_IRQn                = 32,     /*!< I2C1 Error Interrupt                                 */
  SPI1_IRQn                   = 35,     /*!< SPI1 global Interrupt                                */
  USART1_IRQn                 = 37,     /*!< USART1 global Interrupt                              */
  USART2_IRQn                 = 38,     /*!< USART2 global Interrupt                              */
  EXTI15_10_IRQn              = 40,     /*!< External Line[15:10] Interrupts                      */
  RTCAlarm_IRQn               = 41,     /*!< RTC Alarm through EXTI Line Interrupt                */
  USBWakeUp_IRQn              = 42,     /*!< USB Device WakeUp from suspend through EXTI Line Interrupt */    
  FPU_IRQn					  = 43,		/*!< FPU Global Interrupt 								  */ 
  QSPI_IRQn					  = 44,		/*!< FPU Global Interrupt 								  */
#endif /* APM32F10X_LD */  


#ifdef APM32F10X_MD
  ADC1_2_IRQn                 = 18,     /*!< ADC1 and ADC2 global Interrupt                       */
  USB_HP_CAN1_TX_IRQn         = 19,     /*!< USB Device High Priority or CAN1 TX Interrupts       */
  USB_LP_CAN1_RX0_IRQn        = 20,     /*!< USB Device Low Priority or CAN1 RX0 Interrupts       */
  CAN1_RX1_IRQn               = 21,     /*!< CAN1 RX1 Interrupt                                   */
  CAN1_SCE_IRQn               = 22,     /*!< CAN1 SCE Interrupt                                   */
  EXTI9_5_IRQn                = 23,     /*!< External Line[9:5] Interrupts                        */
  TIM1_BRK_IRQn               = 24,     /*!< TIM1 Break Interrupt                                 */
  TIM1_UP_IRQn                = 25,     /*!< TIM1 Update Interrupt                                */
  TIM1_TRG_COM_IRQn           = 26,     /*!< TIM1 Trigger and Commutation Interrupt               */
  TIM1_CC_IRQn                = 27,     /*!< TIM1 Capture Compare Interrupt                       */
  TIM2_IRQn                   = 28,     /*!< TIM2 global Interrupt                                */
  TIM3_IRQn                   = 29,     /*!< TIM3 global Interrupt                                */
  TIM4_IRQn                   = 30,     /*!< TIM4 global Interrupt                                */
  I2C1_EV_IRQn                = 31,     /*!< I2C1 Event Interrupt                                 */
  I2C1_ER_IRQn                = 32,     /*!< I2C1 Error Interrupt                                 */
  I2C2_EV_IRQn                = 33,     /*!< I2C2 Event Interrupt                                 */
  I2C2_ER_IRQn                = 34,     /*!< I2C2 Error Interrupt                                 */
  SPI1_IRQn                   = 35,     /*!< SPI1 global Interrupt                                */
  SPI2_IRQn                   = 36,     /*!< SPI2 global Interrupt                                */
  USART1_IRQn                 = 37,     /*!< USART1 global Interrupt                              */
  USART2_IRQn                 = 38,     /*!< USART2 global Interrupt                              */
  USART3_IRQn                 = 39,     /*!< USART3 global Interrupt                              */
  EXTI15_10_IRQn              = 40,     /*!< External Line[15:10] Interrupts                      */
  RTCAlarm_IRQn               = 41,     /*!< RTC Alarm through EXTI Line Interrupt                */
  USBWakeUp_IRQn              = 42,     /*!< USB Device WakeUp from suspend through EXTI Line Interrupt */  
  FPU_IRQn					  = 43,		/*!< FPU Global Interrupt 								  */   
  QSPI_IRQn					  = 44,		/*!< FPU Global Interrupt 								  */
#endif /* APM32F10X_MD */  

#ifdef APM32F10X_HD
  ADC1_2_IRQn                 = 18,     /*!< ADC1 and ADC2 global Interrupt                       */
  USB_HP_CAN1_TX_IRQn         = 19,     /*!< USB Device High Priority or CAN1 TX Interrupts       */
  USB_LP_CAN1_RX0_IRQn        = 20,     /*!< USB Device Low Priority or CAN1 RX0 Interrupts       */
  CAN1_RX1_IRQn               = 21,     /*!< CAN1 RX1 Interrupt                                   */
  CAN1_SCE_IRQn               = 22,     /*!< CAN1 SCE Interrupt                                   */
  EXTI9_5_IRQn                = 23,     /*!< External Line[9:5] Interrupts                        */
  TIM1_BRK_IRQn               = 24,     /*!< TIM1 Break Interrupt                                 */
  TIM1_UP_IRQn                = 25,     /*!< TIM1 Update Interrupt                                */
  TIM1_TRG_COM_IRQn           = 26,     /*!< TIM1 Trigger and Commutation Interrupt               */
  TIM1_CC_IRQn                = 27,     /*!< TIM1 Capture Compare Interrupt                       */
  TIM2_IRQn                   = 28,     /*!< TIM2 global Interrupt                                */
  TIM3_IRQn                   = 29,     /*!< TIM3 global Interrupt                                */
  TIM4_IRQn                   = 30,     /*!< TIM4 global Interrupt                                */
  I2C1_EV_IRQn                = 31,     /*!< I2C1 Event Interrupt                                 */
  I2C1_ER_IRQn                = 32,     /*!< I2C1 Error Interrupt                                 */
  I2C2_EV_IRQn                = 33,     /*!< I2C2 Event Interrupt                                 */
  I2C2_ER_IRQn                = 34,     /*!< I2C2 Error Interrupt                                 */
  SPI1_IRQn                   = 35,     /*!< SPI1 global Interrupt                                */
  SPI2_IRQn                   = 36,     /*!< SPI2 global Interrupt                                */
  USART1_IRQn                 = 37,     /*!< USART1 global Interrupt                              */
  USART2_IRQn                 = 38,     /*!< USART2 global Interrupt                              */
  USART3_IRQn                 = 39,     /*!< USART3 global Interrupt                              */
  EXTI15_10_IRQn              = 40,     /*!< External Line[15:10] Interrupts                      */
  RTCAlarm_IRQn               = 41,     /*!< RTC Alarm through EXTI Line Interrupt                */
  USBWakeUp_IRQn              = 42,     /*!< USB Device WakeUp from suspend through EXTI Line Interrupt */
  TIM8_BRK_IRQn               = 43,     /*!< TIM8 Break Interrupt                                 */
  TIM8_UP_IRQn                = 44,     /*!< TIM8 Update Interrupt                                */
  TIM8_TRG_COM_IRQn           = 45,     /*!< TIM8 Trigger and Commutation Interrupt               */
  TIM8_CC_IRQn                = 46,     /*!< TIM8 Capture Compare Interrupt                       */
  ADC3_IRQn                   = 47,     /*!< ADC3 global Interrupt                                */
  FSMC_IRQn                   = 48,     /*!< FSMC global Interrupt                                */
  SDIO_IRQn                   = 49,     /*!< SDIO global Interrupt                                */
  TIM5_IRQn                   = 50,     /*!< TIM5 global Interrupt                                */
  SPI3_IRQn                   = 51,     /*!< SPI3 global Interrupt                                */
  UART4_IRQn                  = 52,     /*!< UART4 global Interrupt                               */
  UART5_IRQn                  = 53,     /*!< UART5 global Interrupt                               */
  TIM6_IRQn                   = 54,     /*!< TIM6 global Interrupt                                */
  TIM7_IRQn                   = 55,     /*!< TIM7 global Interrupt                                */
  DMA2_Channel1_IRQn          = 56,     /*!< DMA2 Channel 1 global Interrupt                      */
  DMA2_Channel2_IRQn          = 57,     /*!< DMA2 Channel 2 global Interrupt                      */
  DMA2_Channel3_IRQn          = 58,     /*!< DMA2 Channel 3 global Interrupt                      */
  DMA2_Channel4_5_IRQn        = 59      /*!< DMA2 Channel 4 and Channel 5 global Interrupt        */
#endif /* APM32F10X_HD */       
} IRQn_Type;

/**
  * @}
  */

#include "core_cm3.h"
#include "system_apm32f10x.h"
#include <stdint.h>


typedef int32_t  s32;
typedef int16_t s16;
typedef int8_t  s8;

typedef const int32_t sc32;
typedef const int16_t sc16;
typedef const int8_t sc8;

typedef __IO int32_t  vs32;
typedef __IO int16_t  vs16;
typedef __IO int8_t   vs8;

typedef __I int32_t vsc32;
typedef __I int16_t vsc16;
typedef __I int8_t vsc8;

typedef uint32_t  u32;
typedef uint16_t u16;
typedef uint8_t  u8;

typedef const uint32_t uc32;
typedef const uint16_t uc16;
typedef const uint8_t uc8; 

typedef __IO uint32_t  vu32;
typedef __IO uint16_t vu16;
typedef __IO uint8_t  vu8;

typedef __I uint32_t vuc32;
typedef __I uint16_t vuc16;
typedef __I uint8_t vuc8;

#ifndef __IM                                    /*!< Fallback for older CMSIS versions                                         */
  #define __IM   __I
#endif
#ifndef __OM                                    /*!< Fallback for older CMSIS versions                                         */
  #define __OM   __O
#endif
#ifndef __IOM                                   /*!< Fallback for older CMSIS versions                                         */
  #define __IOM  __IO
#endif

enum {BIT_RESET, BIT_SET};
enum {RESET, SET};
enum {DISABLE, ENABLE};
enum {ERROR, SUCCESS};

#ifndef NULL 
#define NULL	((void *)0)
#endif

#if defined (__CC_ARM )
#pragma anon_unions
#endif

/**
  * @brief Reset and clock management unit (RCM)
  */
typedef struct 
{                               
	/** Clock control register */
	union 
	{
		__IOM uint32_t CTRL;                         

		struct 
		{
			__IOM uint32_t HIRCEN		: 1;           
		  	__IM  uint32_t HIRCRDYF		: 1;           
		  	__IM  uint32_t RESERVED1  	: 1;
		  	__IOM uint32_t HIRCTRIM		: 5;           
		  	__IM  uint32_t HIRCCAL		: 8;           
		  	__IOM uint32_t HXTEN      	: 1;           
		  	__IM  uint32_t HXTRDYF		: 1;           
		  	__IOM uint32_t HXTBYP     	: 1;           
		  	__IOM uint32_t CSSEN      	: 1;           
		  	__IM  uint32_t RESERVED2  	: 4;
		  	__IOM uint32_t PLLEN      	: 1;           
		  	__IM  uint32_t PLLRDYF    	: 1; 
		  	__IM  uint32_t RESERVED3  	: 6;
		} CTRL_B;
 	} ;
		  
	/** Clock configuration register */
	union 
	{
		__IOM uint32_t CFG;                       
	
		struct 
		{
			__IOM uint32_t SCSEL		: 2;           
			__IM  uint32_t SCS        	: 2;           
			__IOM uint32_t AHBDIV     	: 4;           
			__IOM uint32_t APB1DIV    	: 3;           
			__IOM uint32_t APB2DIV    	: 3;           
			__IOM uint32_t ADCDIV     	: 2;           
			__IOM uint32_t PLLSEL     	: 1;           
			__IOM uint32_t PLLXTDIV   	: 1;           
			__IOM uint32_t PLLMF      	: 4;           
			__IOM uint32_t USBDIV   	: 1;           
			__IM  uint32_t RESERVED1  	: 1;
			__IOM uint32_t COC        	: 3;           
			__IOM uint32_t FPUDIV		: 1;	
			__IM  uint32_t RESERVED2  	: 4;
		} CFG_B;
	} ;
	  
	/** Clock interrupt control register */
	union 
	{
		__IOM uint32_t INT;                        

		struct 
		{
			__IM  uint32_t LIRCRDYIF	: 1;           
			__IM  uint32_t LXTRDYIF		: 1;           
			__IM  uint32_t HIRCRDYIF	: 1;           
			__IM  uint32_t HXTRDYIF		: 1;           
			__IM  uint32_t PLLRDYIF		: 1;           
			__IM  uint32_t RESERVED1	: 2;
			__IM  uint32_t CSSIF		: 1;           
			__IOM uint32_t LIRCRDYIE	: 1;           
			__IOM uint32_t LXTRDYIE		: 1;           
			__IOM uint32_t HIRCRDYIE	: 1;           
			__IOM uint32_t HXTRDYIE		: 1;           
			__IOM uint32_t PLLRDYIE		: 1;           
			__IM  uint32_t RESERVED2	: 3;
			__OM  uint32_t LIRCRDYC		: 1;           
			__OM  uint32_t LXTRDYC		: 1;           
			__OM  uint32_t HIRCRDYC		: 1;           
			__OM  uint32_t HXTRDYC		: 1;           
			__OM  uint32_t PLLRDYC		: 1;           
			__IM  uint32_t RESERVED3	: 2;
			__OM  uint32_t CSSIFC       : 1; 
			__IM  uint32_t RESERVED4	: 8;
		} INT_B;
	} ;
  
	/** APB2 peripheral reset register */
	union 
	{
		__IOM uint32_t APB2RST;                   

		struct 
		{
			__IOM uint32_t AFIORST			: 1;           
			__IM  uint32_t RESERVED1		: 1;
			__IOM uint32_t PARST			: 1;           
			__IOM uint32_t PBRST			: 1;           
			__IOM uint32_t PCRST			: 1;           
			__IOM uint32_t PDRST			: 1;           
			__IOM uint32_t PERST			: 1;           
			__IOM uint32_t PFRST			: 1;           
			__IOM uint32_t PGRST			: 1;           
			__IOM uint32_t ADC1RST			: 1;           
			__IOM uint32_t ADC2RST			: 1;           
			__IOM uint32_t TMR1RST			: 1;           
			__IOM uint32_t SPI1RST			: 1;           
			__IOM uint32_t TMR8RST			: 1;           
			__IOM uint32_t USART1RST		: 1;           
			__IOM uint32_t ADC3RST			: 1;           
			__IM  uint32_t RESERVED2		: 16;          
		} APB2RST_B;
	} ;
  
	/** APB1 peripheral reset register */
	union {
		__IOM uint32_t APB1RST;                   

		struct 
		{
			__IOM uint32_t TMR2RST	: 1;           
			__IOM uint32_t TMR3RST    : 1;           
			__IOM uint32_t TMR4RST    : 1;           
			__IOM uint32_t TMR5RST    : 1;           
			__IOM uint32_t TMR6RST    : 1;           
			__IOM uint32_t TMR7RST    : 1;           
			__IOM uint32_t RESERVED1  : 5;           
			__IOM uint32_t WWDTRST    : 1;           
			__IM  uint32_t RESERVED2 	: 2;
			__IOM uint32_t SPI2RST    : 1;           
			__IOM uint32_t SPI3RST    : 1;           
			__IM  uint32_t RESERVED3  : 1;
			__IOM uint32_t USART2RST  : 1;           
			__IOM uint32_t USART3RST  : 1;           
			__IOM uint32_t UART4RST   : 1;           
			__IOM uint32_t UART5RST   : 1;           
			__IOM uint32_t I2C1RST    : 1;           
			__IOM uint32_t I2C2RST    : 1;           
			__IOM uint32_t USBRST     : 1;           
			__IM  uint32_t RESERVED4  : 1;
			__IOM uint32_t CANRST     : 1;           
			__IM  uint32_t RESERVED5  : 1;
			__IOM uint32_t BKPRST     : 1;           
			__IOM uint32_t PMURST     : 1;           
			__IOM uint32_t DACRST     : 1;
			__IM  uint32_t RESERVED6  : 2;
		} APB1RST_B;
	} ;

	/** AHB clock enable register */
	union 
	{
		__IOM uint32_t AHBCLKEN;                     

		struct 
		{
			__IOM uint32_t DMA1CLKE			: 1;           
			__IOM uint32_t DMA2CLKE     	: 1;           
			__IOM uint32_t SRAMCLKE     	: 1;           
			__IOM uint32_t FPUCLKE      	: 1;			
			__IOM uint32_t FMCCLKE	  		: 1;           
			__IOM uint32_t QSPICLKE     	: 1;			
			__IOM uint32_t CRCCLKE      	: 1;           
			__IM  uint32_t RESERVED1    	: 1;
			__IOM uint32_t EMMCCLKEN    	: 1;           
			__IM  uint32_t RESERVED2    	: 1;
			__IOM uint32_t SDIOCLKE     	: 1; 
			__IM  uint32_t RESERVED3    	: 21;
		} AHBCLKEN_B;
	} ;

	/** APB2 clock enable register */
	union 
	{
		__IOM uint32_t APB2CLKEN;                    

		struct 
		{
			__IOM uint32_t AFIOCLKE			: 1;           
			__IM  uint32_t RESERVED1     	: 1;
			__IOM uint32_t PACLKE     		: 1;           
			__IOM uint32_t PBCLKE     		: 1;           
			__IOM uint32_t PCCLKE     		: 1;           
			__IOM uint32_t PDCLKE     		: 1;           
			__IOM uint32_t PECLKE     		: 1;           
			__IOM uint32_t PFCLKE     		: 1;           
			__IOM uint32_t PGCLKE     		: 1;           
			__IOM uint32_t ADC1CLKE     	: 1;           
			__IOM uint32_t ADC2CLKE     	: 1;           
			__IOM uint32_t TMR1CLKE     	: 1;           
			__IOM uint32_t SPI1CLKE     	: 1;           
			__IOM uint32_t TMR8CLKE     	: 1;           
			__IOM uint32_t USART1CLKE   	: 1;           
			__IOM uint32_t ADC3CLKE     	: 1;           
			__IM  uint32_t RESERVED2   		: 16;        
		} APB2CLKEN_B;
	};
  
	/** APB1 clock enable register */
	union 
	{
		__IOM uint32_t APB1CLKEN;                    

		struct 
		{
			__IOM uint32_t TMR2CLKE     	: 1;           
			__IOM uint32_t TMR3CLKE     	: 1;           
			__IOM uint32_t TMR4CLKE     	: 1;           
			__IOM uint32_t TMR5CLKE     	: 1;           
			__IOM uint32_t TMR6CLKE     	: 1;           
			__IOM uint32_t TMR7CLKE    		: 1;                      
			__IM  uint32_t RESERVED1   		: 5;
			__IOM uint32_t WWDTCLKE     	: 1;           
			__IM  uint32_t RESERVED2 		: 2;
			__IOM uint32_t SPI2CLKE     	: 1;           
			__IOM uint32_t SPI3CLKE     	: 1;           
			__IM  uint32_t RESERVED3   		: 1;
			__IOM uint32_t USART2CLKE   	: 1;           
			__IOM uint32_t USART3CLKE   	: 1;           
			__IOM uint32_t UART4CLKE    	: 1;           
			__IOM uint32_t UART5CLKE    	: 1;           
			__IOM uint32_t I2C1CLKE     	: 1;           
			__IOM uint32_t I2C2CLKE     	: 1;           
			__IOM uint32_t USBCLKE      	: 1;           
			__IM  uint32_t RESERVED4    	: 1;
			__IOM uint32_t CANCLKE      	: 1;           
			__IM  uint32_t RESERVED5		: 1;
			__IOM uint32_t BKPCLKE      	: 1;           
			__IOM uint32_t PMUCLKE      	: 1;           
			__IOM uint32_t DACCLKE      	: 1;    
			__IM  uint32_t RESERVED6   		: 2;
		} APB1CLKEN_B;
	} ;
  
	/** Backup domain control register */
	union 
	{
		__IOM uint32_t BD;                       

		struct 
		{
			__IOM uint32_t LXTEN      		: 1;           
			__IM  uint32_t LXTRDYF     		: 1;           
			__IOM uint32_t LXTBYP     		: 1;           
			__IM  uint32_t RESERVED1    	: 5;
			__IOM uint32_t RTCSEL     		: 2;           
			__IM  uint32_t RESERVED2    	: 5;
			__IOM uint32_t RTCEN      		: 1;           
			__IOM uint32_t BDRST      		: 1;  
			__IM  uint32_t RESERVED3    	: 15;
		} BD_B;
	} ;
  
	/** Control/status register */
	union 
	{
		__IOM uint32_t CSTS;                        

		struct 
		{
			__IOM uint32_t LIRCEN      		: 1;           
			__IM  uint32_t LIRCRDYF     	: 1;           
			__IM  uint32_t RESERVED1  		: 22;
			__IOM uint32_t CLRRSTF       	: 1;           
			__IM  uint32_t RESERVED2 		: 1;
			__IOM uint32_t PINRSTF    		: 1;           
			__IOM uint32_t PWRRSTF    		: 1;           
			__IOM uint32_t SWRSTF    		: 1;           
			__IOM uint32_t IWDTRSTF   		: 1;           
			__IOM uint32_t WWDTRSTF   		: 1;           
			__IOM uint32_t LPRRSTF   		: 1;           
		} CSTS_B;
	} ;
} RCM_T;                                     	


/**
  * @brief General purpose I/O (GPIO)
  */
typedef struct 
{                               
  
	/** Port control register0 */
	union
	{
		__IOM uint32_t CTRL0;                        

		struct
		{
			__IOM uint32_t MOD0  		: 2;           
			__IOM uint32_t CFG0       	: 2;           
			__IOM uint32_t MOD1      	: 2;           
			__IOM uint32_t CFG1       	: 2;           
			__IOM uint32_t MOD2      	: 2;           
			__IOM uint32_t CFG2       	: 2;           
			__IOM uint32_t MOD3      	: 2;           
			__IOM uint32_t CFG3       	: 2;           
			__IOM uint32_t MOD4      	: 2;           
			__IOM uint32_t CFG4       	: 2;           
			__IOM uint32_t MOD5      	: 2;           
			__IOM uint32_t CFG5       	: 2;           
			__IOM uint32_t MOD6      	: 2;           
			__IOM uint32_t CFG6       	: 2;           
			__IOM uint32_t MOD7      	: 2;           
			__IOM uint32_t CFG7       	: 2;           
		} CTRL0_B;
	} ;

	/** Port control register1 */
	union 
	{
		__IOM uint32_t CTRL1;                        

		struct 
		{
			__IOM uint32_t MOD8      	: 2;           
			__IOM uint32_t CFG8       	: 2;           
			__IOM uint32_t MOD9      	: 2;           
			__IOM uint32_t CFG9       	: 2;           
			__IOM uint32_t MOD10     	: 2;           
			__IOM uint32_t CFG10      	: 2;           
			__IOM uint32_t MOD11     	: 2;           
			__IOM uint32_t CFG11      	: 2;           
			__IOM uint32_t MOD12     	: 2;           
			__IOM uint32_t CFG12      	: 2;           
			__IOM uint32_t MOD13     	: 2;           
			__IOM uint32_t CFG13      	: 2;           
			__IOM uint32_t MOD14     	: 2;           
			__IOM uint32_t CFG14      	: 2;           
			__IOM uint32_t MOD15     	: 2;           
			__IOM uint32_t CFG15      	: 2;           
		} CTRL1_B;
	} ;

  	/** Port data in register */
	union 
	{
		__IM  uint32_t DIN;                        

		struct 
		{
			__IM  uint32_t DIN0       	: 1;           
			__IM  uint32_t DIN1       	: 1;           
			__IM  uint32_t DIN2       	: 1;           
			__IM  uint32_t DIN3       	: 1;           
			__IM  uint32_t DIN4       	: 1;           
			__IM  uint32_t DIN5       	: 1;           
			__IM  uint32_t DIN6       	: 1;           
			__IM  uint32_t DIN7       	: 1;           
			__IM  uint32_t DIN8       	: 1;           
			__IM  uint32_t DIN9       	: 1;           
			__IM  uint32_t DIN10      	: 1;           
			__IM  uint32_t DIN11      	: 1;           
			__IM  uint32_t DIN12      	: 1;           
			__IM  uint32_t DIN13      	: 1;           
			__IM  uint32_t DIN14      	: 1;           
			__IM  uint32_t DIN15      	: 1;   
			__IM  uint32_t RESERVED		: 16;
		} DIN_B;
	} ;
	  
	/** Port data output register */
	union 
	{
		__IOM uint32_t DOUT;                        

		struct 
		{
			__IOM uint32_t DOUT0       	: 1;           
			__IOM uint32_t DOUT1       	: 1;           
			__IOM uint32_t DOUT2       	: 1;           
			__IOM uint32_t DOUT3       	: 1;           
			__IOM uint32_t DOUT4       	: 1;           
			__IOM uint32_t DOUT5       	: 1;           
			__IOM uint32_t DOUT6       	: 1;           
			__IOM uint32_t DOUT7       	: 1;           
			__IOM uint32_t DOUT8       	: 1;           
			__IOM uint32_t DOUT9       	: 1;           
			__IOM uint32_t DOUT10      	: 1;           
			__IOM uint32_t DOUT11      	: 1;           
			__IOM uint32_t DOUT12      	: 1;           
			__IOM uint32_t DOUT13      	: 1;           
			__IOM uint32_t DOUT14      	: 1;           
			__IOM uint32_t DOUT15      	: 1;    
			__IM  uint32_t RESERVED		: 16;
		} DOUT_B;
	} ;
  
	/** Port bit set/clear register */
	union 
	{
		__OM  uint32_t BSC;                       
			
		struct 
		{
			__OM  uint32_t BS0        : 1;           
			__OM  uint32_t BS1        : 1;           
			__OM  uint32_t BS2        : 1;           
			__OM  uint32_t BS3        : 1;           
			__OM  uint32_t BS4        : 1;           
			__OM  uint32_t BS5        : 1;           
			__OM  uint32_t BS6        : 1;           
			__OM  uint32_t BS7        : 1;           
			__OM  uint32_t BS8        : 1;           
			__OM  uint32_t BS9        : 1;           
			__OM  uint32_t BS10       : 1;           
			__OM  uint32_t BS11       : 1;           
			__OM  uint32_t BS12       : 1;           
			__OM  uint32_t BS13       : 1;           
			__OM  uint32_t BS14       : 1;           
			__OM  uint32_t BS15       : 1;           
			__OM  uint32_t BR0        : 1;           
			__OM  uint32_t BC1        : 1;           
			__OM  uint32_t BC2        : 1;           
			__OM  uint32_t BR3        : 1;           
			__OM  uint32_t BC4        : 1;           
			__OM  uint32_t BC5        : 1;           
			__OM  uint32_t BC6        : 1;           
			__OM  uint32_t BC7        : 1;           
			__OM  uint32_t BC8        : 1;           
			__OM  uint32_t BC9        : 1;           
			__OM  uint32_t BC10       : 1;           
			__OM  uint32_t BC11       : 1;           
			__OM  uint32_t BC12       : 1;           
			__OM  uint32_t BC13       : 1;           
			__OM  uint32_t BC14       : 1;           
			__OM  uint32_t BC15       : 1;           
		} BSC_B;
	} ;
  
	/** Port bit clear register */
	union 
	{
		__OM  uint32_t BC;                        

		struct 
		{
			__OM  uint32_t BC0   		: 1;           
			__OM  uint32_t BC1        	: 1;           
			__OM  uint32_t BC2        	: 1;           
			__OM  uint32_t BC3        	: 1;           
			__OM  uint32_t BC4        	: 1;           
			__OM  uint32_t BC5        	: 1;           
			__OM  uint32_t BC6        	: 1;           
			__OM  uint32_t BC7        	: 1;           
			__OM  uint32_t BC8        	: 1;           
			__OM  uint32_t BC9        	: 1;           
			__OM  uint32_t BC10       	: 1;           
			__OM  uint32_t BC11       	: 1;           
			__OM  uint32_t BC12       	: 1;           
			__OM  uint32_t BC13       	: 1;           
			__OM  uint32_t BC14       	: 1;           
			__OM  uint32_t BC15       	: 1;   
			__IM  uint32_t RESERVED		: 16;
		} BC_B;
	} ;
  
	/** Port configuration lock register */
	union {
		__IOM uint32_t LOCK;                        

		struct 
		{
			__IOM uint32_t LCK0       	: 1;            
			__IOM uint32_t LCK1       	: 1;            
			__IOM uint32_t LCK2       	: 1;            
			__IOM uint32_t LCK3       	: 1;            
			__IOM uint32_t LCK4       	: 1;            
			__IOM uint32_t LCK5       	: 1;            
			__IOM uint32_t LCK6       	: 1;            
			__IOM uint32_t LCK7       	: 1;            
			__IOM uint32_t LCK8       	: 1;            
			__IOM uint32_t LCK9       	: 1;            
			__IOM uint32_t LCK10      	: 1;            
			__IOM uint32_t LCK11      	: 1;            
			__IOM uint32_t LCK12      	: 1;            
			__IOM uint32_t LCK13      	: 1;            
			__IOM uint32_t LCK14      	: 1;            
			__IOM uint32_t LCK15      	: 1;            
			__IOM uint32_t LCKK			: 1;      
			__IM  uint32_t RESERVED		: 16;
	} LOCK_B;
  } ;
} GPIO_T;                                   	

/**
  * @brief Alternate function I/O (AFIO)
  */
typedef struct 
{                                
	/** Event control register */
	union 
	{
		__IOM uint32_t EVCTRL;                        

		struct 
		{
			__IOM uint32_t PINSEL			: 4;            
			__IOM uint32_t PORTSEL  		: 3;            
			__IOM uint32_t EVOEN  			: 1; 
			__IM  uint32_t RESERVED			: 24;
		} EVCTRL_B;
	} ;

  	/** Alternate function IO remap and Serial wire JTAG configuration register */
	union {
		__IOM uint32_t REMAP;                        
		                                        
		struct 
		{
			__IOM uint32_t SPI1_REMAP			: 1;            
			__IOM uint32_t I2C1_REMAP 			: 1;            
			__IOM uint32_t USART1_REMAP 		: 1;          
			__IOM uint32_t USART2_REMAP 		: 1;          
			__IOM uint32_t USART3_REMAP 		: 2;          
			__IOM uint32_t TMR1_REMAP 			: 2;            
			__IOM uint32_t TMR2_REMAP 			: 2;            
			__IOM uint32_t TMR3_REMAP 			: 2;            
			__IOM uint32_t TMR4_REMAP 			: 1;            
			__IOM uint32_t CAN_REMAP  			: 2;            
			__IOM uint32_t PD01_REMAP 			: 1;            
			__IOM uint32_t TMR5_CH4_REMAP 		: 1;        
			__IOM uint32_t ADC1_ETI_REMAP 		: 1;    
			__IOM uint32_t ADC1_ETR_REMAP 		: 1;    
			__IOM uint32_t ADC2_ETI_REMAP 		: 1;    
			__IOM uint32_t ADC2_ETR_REMAP 		: 1;    
			__IM  uint32_t RESERVED1        	: 3;
			__OM  uint32_t SWJ_CFG    			: 3; 
			__IM  uint32_t RESERVED2        	: 5;
		} REMAP_B;
	} ;

	/** External interrupt configuration register1 */   
	union {
		__IOM uint32_t EINTCFG1;                     

		struct 
		{
			__IOM uint32_t EINT0      	: 4;            
			__IOM uint32_t EINT1      	: 4;            
			__IOM uint32_t EINT2      	: 4;            
			__IOM uint32_t EINT3      	: 4; 
			__IM  uint32_t RESERVED		: 16;
		} EINTCFG1_B;
	} ;

	/** External interrupt configuration register2 */   
	union {
		__IOM uint32_t EINTCFG2;                     

		struct 
		{
			__IOM uint32_t EINT4      	: 4;            
			__IOM uint32_t EINT5      	: 4;            
			__IOM uint32_t EINT6      	: 4;            
			__IOM uint32_t EINT7      	: 4;  
			__IM  uint32_t RESERVED		: 16;
		} EINTCFG2_B;
	} ;

	/** External interrupt configuration register3 */ 	  
	union 
	{
		__IOM uint32_t EINTCFG3;                     

		struct 
		{
			__IOM uint32_t EINT8      	: 4;            
			__IOM uint32_t EINT9      	: 4;            
			__IOM uint32_t EINT10     	: 4;            
			__IOM uint32_t EINT11     	: 4;  
			__IM  uint32_t RESERVED		: 16;
		} EINTCFG3_b;
	} ;
	
	/** External interrupt configuration register4 */  
	union 
	{
		__IOM uint32_t EINTCFG4;                     

		struct 
		{
			__IOM uint32_t EINT12     	: 4;            
			__IOM uint32_t EINT13     	: 4;            
			__IOM uint32_t EINT14     	: 4;            
			__IOM uint32_t EINT15     	: 4;   
			__IM  uint32_t RESERVED		: 16;
		} EINTCFG4_b;
	} ;
	__IM  uint32_t  RESERVED;
  
	/** Alternate function IO remap register2 */
	union 
	{
		__IOM uint32_t REMAP2;                       

		struct 
		{
			__IM  uint32_t RESERVED1		: 5;
			__IOM uint32_t TMR9_REMAP 		: 1;            
			__IOM uint32_t TMR10_REMAP 		: 1;           
			__IOM uint32_t TMR11_REMAP 		: 1;           
			__IOM uint32_t TMR13_REMAP 		: 1;           
			__IOM uint32_t TMR14_REMAP 		: 1;           
			__IOM uint32_t EMMC_NADV  		: 1; 
			__IM  uint32_t RESERVED2		: 21;
		} REMAP2_B;
	} ;
} AFIO_T;                                    	

/**
  * @brief Universal synchronous asynchronous receiver  transmitter (USART)
  */
typedef struct 
{                               
  
	/** Status register */
	union 
	{
		__IOM uint32_t STS;                         

		struct 
		{
			__IM  uint32_t PEF         	: 1;           
			__IM  uint32_t FEF         	: 1;           
			__IM  uint32_t NEF         	: 1;           
			__IM  uint32_t OVREF        : 1;           
			__IM  uint32_t IDLEF       	: 1;           
			__IOM uint32_t RXBNEF       : 1;           
			__IOM uint32_t TXCF         : 1;           
			__IM  uint32_t TXBEF        : 1;           
			__IOM uint32_t LBDF        	: 1;           
			__IOM uint32_t CTSF   		: 1;  
			__IM  uint32_t RESERVED		: 22;
		} STS_B;
	} ;
  
	/** TX Buffer Data Register */
	union 
	{
		__IOM uint32_t DATA;                         

		struct 
		{
			__IOM uint32_t DATA         	: 9;   
			__IM  uint32_t RESERVED			: 23;
		} DATA_B;
	} ;
  
	/** Baud rate register */
	union 
	{
		__IOM uint32_t BR;                        

		struct 
		{
			__IOM uint32_t FRACTION			: 4;      
			__IOM uint32_t MANTISSA			: 12;     
			__IM  uint32_t RESERVED			: 16;
		} BR_B;
	} ;
  
	/** Control register 1 */
	union 
	{
	
		__IOM uint32_t CTRL1;                        

		struct 
		{
			__IOM uint32_t TXBK    		: 1;           
			__IOM uint32_t RXWUP        : 1;           
			__IOM uint32_t RXEN         : 1;           
			__IOM uint32_t TXEN         : 1;           
			__IOM uint32_t IDLEIE     	: 1;           
			__IOM uint32_t RXBNEIE     	: 1;           
			__IOM uint32_t TXCIE       	: 1;           
			__IOM uint32_t TXBEIE      	: 1;           
			__IOM uint32_t PEIE       	: 1;           
			__IOM uint32_t PMSEL       	: 1;           
			__IOM uint32_t PCEN        	: 1;           
			__IOM uint32_t WUPM       	: 1;           
			__IOM uint32_t WLS          : 1;           
			__IOM uint32_t UEN         	: 1;    
			__IM  uint32_t RESERVED		: 18;
		} CTRL1_B;
	} ;

  	/** Control register 2 */
	union 
	{
		__IOM uint32_t CTRL2;                        

		struct 
		{
			__IOM uint32_t ADDR        	: 4;           
			__IM  uint32_t RESERVED1    : 1;
			__IOM uint32_t LBDL       	: 1;           
			__IOM uint32_t LBDIE      	: 1;           
			__IM  uint32_t RESERVED2   	: 1;
			__IOM uint32_t LBCEN       	: 1;           
			__IOM uint32_t CLKPHA       : 1;           
			__IOM uint32_t CLKPOL       : 1;           
			__IOM uint32_t CKPEN      	: 1;           
			__IOM uint32_t STOPB       	: 2;           
			__IOM uint32_t LINEN      	: 1;  
			__IM  uint32_t RESERVED3   	: 17;
		} CTRL2_B;
	} ;
  
	/** Control register 3 */
	union 
	{
		__IOM uint32_t CTRL3;                        

		struct 
		{
			__IOM uint32_t ERRIE        : 1;           
			__IOM uint32_t IRDAEN       : 1;           
			__IOM uint32_t IRDALP       : 1;           
			__IOM uint32_t HDEN      	: 1;           
			__IOM uint32_t NACKEN       : 1;           
			__IOM uint32_t SCEN       	: 1;           
			__IOM uint32_t RXDMAEN     	: 1;           
			__IOM uint32_t TXDMAEN     	: 1;           
			__IOM uint32_t RTSEN       	: 1;           
			__IOM uint32_t CTSEN       	: 1;           
			__IOM uint32_t CTSIE      	: 1; 
			__IM  uint32_t RESERVED   	: 21;
		} CTRL3_B;
	} ;
  
	/** Guard time and divider number register */
	union 
	{
		__IOM uint32_t GTDIV;                       

		struct 
		{
			__IOM uint32_t DIV        	: 8;           
			__IOM uint32_t GT         	: 8; 
			__IM  uint32_t RESERVED   	: 16;
		} GTDIV_B;
	} ;
} USART_T;                                  	

/**
  * @brief Flash memory controller(FMC)
  */
typedef struct 
{                               
	/** Flash access control register */
	union 
	{
		__IOM uint32_t ACTRL;                        

		struct 
		{
			__IOM uint32_t LATENCY   	: 3;           
			__IOM uint32_t HLFCYEN     	: 1;           
			__IOM uint32_t PBEN     	: 1;           
			__IM  uint32_t PBS     		: 1;    
			__IM  uint32_t RESERVED   	: 26;
		} ACTRL_B;
	} ;

	/** key register */
	union 
	{
		__OM  uint32_t KEY;                       

		struct 
		{
			__OM  uint32_t KEY        : 32;          
		} KEY_B;
	} ;

	/** option byte key register */  
	union 
	{
		__OM  uint32_t OBEYR;                    

		struct 
		{
			__OM  uint32_t OBEYR     : 32;          
		} OBEYR_B;
	};
  
	/** status register */
	union 
	{
		__IOM uint32_t STS;                         

		struct 
		{
			__IM  uint32_t BUSY        		: 1;           
			__IM  uint32_t RESERVED1    	: 1;
			__IOM uint32_t PGERR      		: 1;           
			__IM  uint32_t RESERVED2    	: 1;
			__IOM uint32_t WPERR   			: 1;           
			__IOM uint32_t EOP        		: 1;   
			__IM  uint32_t RESERVED3    	: 26;
		} STS_B;
	};

	/** status register */  
	union 
	{
		__IOM uint32_t CTRL;                         

		struct 
		{
			__IOM uint32_t PROG         	: 1;           
			__IOM uint32_t PGERA        	: 1;           
			__IOM uint32_t MERA        		: 1;           
			__IM  uint32_t RESERVED1       	: 1;
			__IOM uint32_t OBPROG      		: 1;           
			__IOM uint32_t OBERA      		: 1;           
			__IOM uint32_t START       		: 1;           
			__IOM uint32_t LOCK       		: 1;           
			__IM  uint32_t RESERVED2       	: 1;
			__IOM uint32_t OBWREN     		: 1;           
			__IOM uint32_t ERRIE      		: 1;           
			__IM  uint32_t RESERVED3       	: 1;
			__IOM uint32_t EOPIE      		: 1;     
			__IM  uint32_t RESERVED4        : 19;
		} CTRL_B;
	} ;
  
	/** address register */
	union 
	{
		__OM  uint32_t ADDR;                         

		struct 
		{
			__OM  uint32_t ADDR		: 32;          
		} ADDR_B;
	};
	
	__IM  uint32_t  RESERVED;
  
	/** Option byte register */
	union 
	{
		__IM  uint32_t OB;                        

		struct 
		{
			__IM  uint32_t OBERR     	: 1;           
			__IM  uint32_t RP      		: 1;           
			__IM  uint32_t WDT     		: 1;           
			__IM  uint32_t STOPRST  	: 1;           
			__IM  uint32_t STDBYRST 	: 1;           
			__IM  uint32_t NOTUSED   	: 5;
			__IM  uint32_t DATA0      	: 8;           
			__IM  uint32_t DATA1      	: 8; 
			__IM  uint32_t RESERVED		: 6;
		} OB_B;
	};
  
	/** Write protection register */
	union 
	{
		__IM  uint32_t WP;                       

		struct
		{
			__IM  uint32_t WP    		: 32;          
		} WP_B;
	};
} FMC_T;                                   	



/** FLASH base address in the alias region */
#define FLASH_BASE            ((uint32_t)0x08000000)
/** SRAM base address in the alias region */
#define SRAM_BASE             ((uint32_t)0x20000000)
/** Peripheral base address in the alias region */
#define PERIPH_BASE           ((uint32_t)0x40000000)

/** SRAM base address in the bit-band region */
#define SRAM_BB_BASE          ((uint32_t)0x22000000)
/** Peripheral base address in the bit-band region */
#define PERIPH_BB_BASE        ((uint32_t)0x42000000)

/** FSMC registers base address */
#define FSMC_R_BASE           ((uint32_t)0xA0000000)

/** Peripheral memory map */
#define APB1PERIPH_BASE       PERIPH_BASE
#define APB2PERIPH_BASE       (PERIPH_BASE + 0x10000)
#define AHBPERIPH_BASE        (PERIPH_BASE + 0x20000)

#define TIM2_BASE             (APB1PERIPH_BASE + 0x0000)
#define TIM3_BASE             (APB1PERIPH_BASE + 0x0400)
#define TIM4_BASE             (APB1PERIPH_BASE + 0x0800)
#define TIM5_BASE             (APB1PERIPH_BASE + 0x0C00)
#define TIM6_BASE             (APB1PERIPH_BASE + 0x1000)
#define TIM7_BASE             (APB1PERIPH_BASE + 0x1400)
#define TIM12_BASE            (APB1PERIPH_BASE + 0x1800)
#define TIM13_BASE            (APB1PERIPH_BASE + 0x1C00)
#define TIM14_BASE            (APB1PERIPH_BASE + 0x2000)
#define RTC_BASE              (APB1PERIPH_BASE + 0x2800)
#define WWDG_BASE             (APB1PERIPH_BASE + 0x2C00)
#define IWDG_BASE             (APB1PERIPH_BASE + 0x3000)
#define SPI2_BASE             (APB1PERIPH_BASE + 0x3800)
#define SPI3_BASE             (APB1PERIPH_BASE + 0x3C00)
#define USART2_BASE           (APB1PERIPH_BASE + 0x4400)
#define USART3_BASE           (APB1PERIPH_BASE + 0x4800)
#define UART4_BASE            (APB1PERIPH_BASE + 0x4C00)
#define UART5_BASE            (APB1PERIPH_BASE + 0x5000)
#define I2C1_BASE             (APB1PERIPH_BASE + 0x5400)
#define I2C2_BASE             (APB1PERIPH_BASE + 0x5800)
#define CAN1_BASE             (APB1PERIPH_BASE + 0x6400)
#define CAN2_BASE             (APB1PERIPH_BASE + 0x6800)
#define BKP_BASE              (APB1PERIPH_BASE + 0x6C00)
#define PWR_BASE              (APB1PERIPH_BASE + 0x7000)
#define DAC_BASE              (APB1PERIPH_BASE + 0x7400)
#define CEC_BASE              (APB1PERIPH_BASE + 0x7800)

#define AFIO_BASE             (APB2PERIPH_BASE + 0x0000)
#define EXTI_BASE             (APB2PERIPH_BASE + 0x0400)
#define GPIOA_BASE            (APB2PERIPH_BASE + 0x0800)
#define GPIOB_BASE            (APB2PERIPH_BASE + 0x0C00)
#define GPIOC_BASE            (APB2PERIPH_BASE + 0x1000)
#define GPIOD_BASE            (APB2PERIPH_BASE + 0x1400)
#define GPIOE_BASE            (APB2PERIPH_BASE + 0x1800)
#define GPIOF_BASE            (APB2PERIPH_BASE + 0x1C00)
#define GPIOG_BASE            (APB2PERIPH_BASE + 0x2000)
#define ADC1_BASE             (APB2PERIPH_BASE + 0x2400)
#define ADC2_BASE             (APB2PERIPH_BASE + 0x2800)
#define TIM1_BASE             (APB2PERIPH_BASE + 0x2C00)
#define SPI1_BASE             (APB2PERIPH_BASE + 0x3000)
#define TIM8_BASE             (APB2PERIPH_BASE + 0x3400)
#define USART1_BASE           (APB2PERIPH_BASE + 0x3800)
#define ADC3_BASE             (APB2PERIPH_BASE + 0x3C00)
#define TIM15_BASE            (APB2PERIPH_BASE + 0x4000)
#define TIM16_BASE            (APB2PERIPH_BASE + 0x4400)
#define TIM17_BASE            (APB2PERIPH_BASE + 0x4800)
#define TIM9_BASE             (APB2PERIPH_BASE + 0x4C00)
#define TIM10_BASE            (APB2PERIPH_BASE + 0x5000)
#define TIM11_BASE            (APB2PERIPH_BASE + 0x5400)

#define SDIO_BASE             (PERIPH_BASE + 0x18000)

#define DMA1_BASE             (AHBPERIPH_BASE + 0x0000)
#define DMA1_Channel1_BASE    (AHBPERIPH_BASE + 0x0008)
#define DMA1_Channel2_BASE    (AHBPERIPH_BASE + 0x001C)
#define DMA1_Channel3_BASE    (AHBPERIPH_BASE + 0x0030)
#define DMA1_Channel4_BASE    (AHBPERIPH_BASE + 0x0044)
#define DMA1_Channel5_BASE    (AHBPERIPH_BASE + 0x0058)
#define DMA1_Channel6_BASE    (AHBPERIPH_BASE + 0x006C)
#define DMA1_Channel7_BASE    (AHBPERIPH_BASE + 0x0080)
#define DMA2_BASE             (AHBPERIPH_BASE + 0x0400)
#define DMA2_Channel1_BASE    (AHBPERIPH_BASE + 0x0408)
#define DMA2_Channel2_BASE    (AHBPERIPH_BASE + 0x041C)
#define DMA2_Channel3_BASE    (AHBPERIPH_BASE + 0x0430)
#define DMA2_Channel4_BASE    (AHBPERIPH_BASE + 0x0444)
#define DMA2_Channel5_BASE    (AHBPERIPH_BASE + 0x0458)
#define RCC_BASE              (AHBPERIPH_BASE + 0x1000)
#define CRC_BASE              (AHBPERIPH_BASE + 0x3000)

/** Flash registers base address */
#define FLASH_R_BASE          (AHBPERIPH_BASE + 0x2000)
/** Flash Option Bytes base address */
#define OB_BASE               ((uint32_t)0x1FFFF800)

#define ETH_BASE              (AHBPERIPH_BASE + 0x8000)
#define ETH_MAC_BASE          (ETH_BASE)
#define ETH_MMC_BASE          (ETH_BASE + 0x0100)
#define ETH_PTP_BASE          (ETH_BASE + 0x0700)
#define ETH_DMA_BASE          (ETH_BASE + 0x1000)

/** FSMC Bank1 registers base address */
#define FSMC_Bank1_R_BASE     (FSMC_R_BASE + 0x0000)
/** FSMC Bank1E registers base address */
#define FSMC_Bank1E_R_BASE    (FSMC_R_BASE + 0x0104)
/** FSMC Bank2 registers base address */
#define FSMC_Bank2_R_BASE     (FSMC_R_BASE + 0x0060)
/** FSMC Bank3 registers base address */
#define FSMC_Bank3_R_BASE     (FSMC_R_BASE + 0x0080)
/**FSMC Bank4 registers base address  */
#define FSMC_Bank4_R_BASE     (FSMC_R_BASE + 0x00A0)

/** Debug MCU registers base address */
#define DBGMCU_BASE          ((uint32_t)0xE0042000)

#define USART2              ((USART_T *) USART2_BASE)
#define USART3              ((USART_T *) USART3_BASE)
#define UART4               ((USART_T *) UART4_BASE)
#define UART5               ((USART_T *) UART5_BASE)
#define AFIO                ((AFIO_T *) AFIO_BASE)
#define GPIOA               ((GPIO_T *) GPIOA_BASE)
#define GPIOB               ((GPIO_T *) GPIOB_BASE)
#define GPIOC               ((GPIO_T *) GPIOC_BASE)
#define GPIOD               ((GPIO_T *) GPIOD_BASE)
#define GPIOE               ((GPIO_T *) GPIOE_BASE)
#define GPIOF               ((GPIO_T *) GPIOF_BASE)
#define GPIOG               ((GPIO_T *) GPIOG_BASE)
#define USART1              ((USART_T *) USART1_BASE)
#define RCM                 ((RCM_T *) RCC_BASE)
#define FMC					((FMC_T *)FLASH_R_BASE)
#ifdef USE_STDPERIPH_DRIVER
  #include "apm32f10x_conf.h"
#endif

/* Define one bit mask */
#define BIT0    0x00000001
#define BIT1    0x00000002
#define BIT2    0x00000004
#define BIT3    0x00000008
#define BIT4    0x00000010
#define BIT5    0x00000020
#define BIT6    0x00000040
#define BIT7    0x00000080
#define BIT8    0x00000100
#define BIT9    0x00000200
#define BIT10   0x00000400
#define BIT11   0x00000800
#define BIT12   0x00001000
#define BIT13   0x00002000
#define BIT14   0x00004000
#define BIT15   0x00008000
#define BIT16   0x00010000
#define BIT17   0x00020000
#define BIT18   0x00040000
#define BIT19   0x00080000
#define BIT20   0x00100000
#define BIT21   0x00200000
#define BIT22   0x00400000
#define BIT23   0x00800000
#define BIT24   0x01000000
#define BIT25   0x02000000
#define BIT26   0x04000000
#define BIT27   0x08000000
#define BIT28   0x10000000
#define BIT29   0x20000000
#define BIT30   0x40000000
#define BIT31   0x80000000

#define SET_BIT(REG, BIT)     ((REG) |= (BIT))

#define CLEAR_BIT(REG, BIT)   ((REG) &= ~(BIT))

#define READ_BIT(REG, BIT)    ((REG) & (BIT))

#define CLEAR_REG(REG)        ((REG) = (0x0))

#define WRITE_REG(REG, VAL)   ((REG) = (VAL))

#define READ_REG(REG)         ((REG))

#define MODIFY_REG(REG, CLEARMASK, SETMASK)  WRITE_REG((REG), (((READ_REG(REG)) & (~(CLEARMASK))) | (SETMASK)))


#ifdef __cplusplus
}
#endif

#endif /* __APM32F10x_H */

