/*!
 * @file    	APM32_MINI.H
 * @brief	 	This file provides set of firmware functions to manage Leds, push-button and COM ports
 *
 * @details		
 *
 * @version		V1.0.0
 * @author  	
 * @date    	2019-9-3
 *
 */
 
#ifndef __APM32_MINI_H
#define __APM32_MINI_H

#ifdef __cplusplus
 extern "C" {
#endif

#include "apm32f10x.h"
#include "apm32f10x_gpio.h"
#include "apm32f10x_usart.h"
#include "apm32f10x_rcm.h"

/**
 * @brief	LED index 
 */
typedef enum 
{
  LED2 = 0,
  LED3 = 1
}APM32_MINI_LED_T;

/**
 * @brief	BUTTON index 
 */
typedef enum 
{  
  BUTTON1 = 0,
  BUTTON2 = 1
}APM32_MINI_BUTTION_T;

/**
 * @brief	COM index 
 */
typedef enum 
{
  COM1 = 0,
  COM2 = 1
}APM32_MINI_COM_T;

#if !defined (USE_APM32_MINI)
#define USE_APM32_MINI
#endif

/** LED2 */
#define APM32_MINI_LED2_PIN                   		GPIO_PIN_6
#define APM32_MINI_LED2_GPIO_PORT             		GPIOE
#define APM32_MINI_LED2_GPIO_CLK            		RCM_APB2_PERIPH_GPIOE  
 
/** LED3 */
#define APM32_MINI_LED3_PIN                    		GPIO_PIN_5
#define APM32_MINI_LED3_GPIO_PORT            		GPIOE
#define APM32_MINI_LED3_GPIO_CLK               		RCM_APB2_PERIPH_GPIOE  
  
/** BUTTON1 */
#define APM32_MINI_BUTTON1_PIN                 		GPIO_PIN_1
#define APM32_MINI_BUTTON1_GPIO_PORT             	GPIOA
#define APM32_MINI_BUTTON1_GPIO_CLK              	RCM_APB2_PERIPH_GPIOA

/** BUTTON2 */
#define APM32_MINI_BUTTON2_PIN            			GPIO_PIN_0
#define APM32_MINI_BUTTON2_GPIO_PORT        		GPIOA
#define APM32_MINI_BUTTON2_GPIO_CLK  				RCM_APB2_PERIPH_GPIOA  

/** COM1 */
#define APM32_MINI_COM1                        		USART1
#define APM32_MINI_COM1_CLK                    		RCM_APB2_PERIPH_USART1
#define APM32_MINI_COM1_TX_PIN                 		GPIO_PIN_9
#define APM32_MINI_COM1_TX_GPIO_PORT           		GPIOA
#define APM32_MINI_COM1_TX_GPIO_CLK            		RCM_APB2_PERIPH_GPIOA
#define APM32_MINI_COM1_RX_PIN                 		GPIO_PIN_10
#define APM32_MINI_COM1_RX_GPIO_PORT           		GPIOA
#define APM32_MINI_COM1_RX_GPIO_CLK            		RCM_APB2_PERIPH_GPIOA
#define APM32_MINI_COM1_IRQn                   		USART1_IRQn
 
/** COM2 */ 
#define APM32_MINI_COM2                        		USART2
#define APM32_MINI_COM2_CLK                    		RCM_APB1_PERIPH_USART2
#define APM32_MINI_COM2_TX_PIN                 		GPIO_PIN_2
#define APM32_MINI_COM2_TX_GPIO_PORT           		GPIOA
#define APM32_MINI_COM2_TX_GPIO_CLK            		RCM_APB2_PERIPH_GPIOA
#define APM32_MINI_COM2_RX_PIN                 		GPIO_PIN_3
#define APM32_MINI_COM2_RX_GPIO_PORT           		GPIOA
#define APM32_MINI_COM2_RX_GPIO_CLK            		RCM_APB2_PERIPH_GPIOA
#define APM32_MINI_COM2_IRQn                   		USART2_IRQn

void APM_MINI_LedInit(APM32_MINI_LED_T led);
void APM_MINI_LedOn(APM32_MINI_LED_T led);
void APM_MINI_LedOff(APM32_MINI_LED_T led);
void APM_MINI_LedToggle(APM32_MINI_LED_T led);
void APM_MINI_COMInit(APM32_MINI_COM_T com, USART_ConfigStruct_T* USART_InitStruct);
 
#ifdef __cplusplus
}
#endif
  
#endif
